"use strict";
this.name = "HyperspaceHangar";
this.author = "Kenji Takashima";
this.copyright = "(C) 2015 Kenji Takashima.";
this.licence = "CC-BY-NC-SA 3.0";
this.description = "Provides a hangar for your ships across the stars.";

this._currentShip = "";
this._currentName = "";
//Defaults
this._defaultpriceOption = 250000;
this._defaulttimeOption = 1800;
//OXPConfig related options
this._showOptions = false;
this._percentPrice = false;
this._shipPercent = 1;
this._configType = "OXPConfig";
this._cost = 0;
this._tradeIn = 0;
this._svEnabled = false;    // ShipVersion installed?
this._hudHidden;
this._debugLogging = false;

// settings for oxpConfig
this.oxpcSettings = {
    Info: {
        Name: this.name,
        Display: "Hyperspace Hangar",
        InfoB: "Select Options. Changing price and time options is disabled" +
            " by default. Percent price is a relatively new option. You will" +
            " be charged the percentage of your ship price selected.",
        InfoS: "Change the percentage charged for percent price. Each unit is 10%."
    },
    Bool0: {
        Name: "_showOptions",
        Def: false,
        Desc: "Show price and time options. Def: false"
    },
    Bool1: {
        Name: "_percentPrice",
        Def: false,
        Desc: "Enables pricing based on ship price percentage."
    },
    SInt0: {
        Name: "_shipPercent",
        Def: 1,
        Max: 10,
        Desc: "Percent to charge."
    }
};
// settings for Library Config
this._hhSettings = {
    Name: this.name,
    Display: "Settings",
    Alias: "Hyperspace Hanger",
    Alive: "_hhSettings",
    Bool: {
        B0: {
            Name: "_showOptions",
            Def: false,
            Desc: "Show price/time options."
        },
        B1: {
            Name: "_percentPrice",
            Def: false,
            Desc: "Percentage price."
        },
        Info: "0 - Displays price and time options.\n1 - Enables pricing based on ship price percentage."
    },
    SInt: {
        S0: {
            Name: "_shipPercent",
            Def: 1,
            Desc: "Percentage to charge",
            Min: 1,
            Max: 10
        },
        Info: "Percentage value to charge player."
    }
};


this.startUpComplete = function () {
    //first time setup
    if (!missionVariables.hyperSpace_namesStored) {
        missionVariables.hyperSpace_namesStored = JSON.stringify({"0_EXIT": "EMPTY"});
        missionVariables.hyperSpace_shipsStored = JSON.stringify({});
        missionVariables.hyperSpace_priceOption = this._defaultpriceOption;
        missionVariables.hyperSpace_timeOption = this._defaulttimeOption;
    }
    //mission setup
    if (!missionVariables.hyperSpace_mission) {
        missionVariables.hyperSpace_mission = "NOT_ACTIVATED";
    }
    //is OXPconfig installed?
    this._oxpConf_check = true;
    if (!worldScripts["OXPConfig.js"]) this._oxpConf_check = false;
    // is Library Config installed? make it the preferred config type
    this._libConf_check = false;
    if (worldScripts.Lib_Config) {
        worldScripts.Lib_Config._registerSet(this._hhSettings);
        this._configType = "Library Config";
        this._libConf_check = true;
        this._oxpConf_check = false;
    }
    if (this._libConf_check == false && this._oxpConf_check == false) {
        this._showOptions = false;
        this._percentPrice = false;
    }

    //check if ShipVersion is installed (it changes the ship buying screen sequence)
    if (worldScripts.shipversion) this._svEnabled = true;

    //names of ships stored. Used because _shipsStored would give JSON strings
    this._currentNames = JSON.parse(missionVariables.hyperSpace_namesStored);
    if (this._currentNames["1_EXIT"]) delete this._currentNames["1_EXIT"];

    //values of ships stored.
    this._shipsStored = JSON.parse(missionVariables.hyperSpace_shipsStored);
    this.$cleanUp();

    //constants for tests
    this._shipPrice = 0;
    this._actuallyBought = 0;
    this._delay = false;

    //current stored ship value; very important
    this._shipTempKey = 0;

    //self explanatory
    this._oldChoice = 0;

    //current prices and storage time
    this._priceOption = missionVariables.hyperSpace_priceOption;
    this._timeOption = missionVariables.hyperSpace_timeOption;

    //information for the transfer function
    this._oldShips = [];
    this._arrivalTime = {};

    //compatibility with Xenon UI
    var w = worldScripts.XenonUI;
    if (w) w.$addMissionScreenException("HyperspaceHangar-map");
    var wx = worldScripts.XenonReduxUI;
    if (wx) wx.$addMissionScreenException("HyperspaceHangar-map");

    this.$dockedCheck();
}

this.shipDockedWithStation = function () {
    this.$dockedCheck();
}

//mark mission system if in G6 and mission has been given
this.playerEnteredNewGalaxy = function () {
    if (galaxyNumber == 5 && missionVariables.hyperSpace_mission == "DELIVER") {
        mission.markSystem({
            system: 149,
            name: "hyperspaceMission",
            markerColor: "greenColor",
            markerScale: 1.5,
            markerShape: "MARKER_DIAMOND"
        });
    }
}

//save variables for next time
this.playerWillSaveGame = function () {
    this.$cleanUp();
    missionVariables.hyperSpace_namesStored = JSON.stringify(this._currentNames);
    missionVariables.hyperSpace_shipsStored = JSON.stringify(this._shipsStored);
    missionVariables.hyperSpace_priceOption = this._priceOption;
    missionVariables.hyperSpace_timeOption = this._timeOption;
}

//create hangar interface if docked and available. remove it otherwise
this.$dockedCheck = function () {
    if (player.ship.docked && player.ship.dockedStation.hasShipyard && Object.keys(this._shipsStored).length > 0) {
        player.ship.dockedStation.setInterface(this.name, {
            title: "Hyperspace Hangar",
            category: "Services",
            summary: "Provides access to ship storage facilities.",
            callback: this.$storeShips.bind(this)
        });
    } else {
        player.ship.dockedStation.setInterface(this.name, null);
    }
}

this.guiScreenChanged = function (to, from) {
    //store player's ship if replacement is a possibility.
    if (guiScreen == "GUI_SCREEN_SHIPYARD") {
        this._shipTempKey = clock.absoluteSeconds;
        var shipinfo = worldScripts["Ship_Storage_Helper.js"].storeCurrentShip(player.ship);
        var data = JSON.parse(shipinfo);
        data[8].length = 0; // remove any cargo
        data[10].length = 0; // remove any passengers
        data[31] = ""; // remove any cargo in smuggling hold
        this._currentShip = JSON.stringify(data);
        this._currentName = player.ship.displayName;
        this._shipPrice = player.ship.price;
        this._tradeIn = this.$tradeInAmount();
        this._actuallyBought = 1;
    } else {
        //has a ship been bought?
        if (from == "GUI_SCREEN_SHIPYARD" && clock.isAdjusting) {
            this._delay = true;
            return;
        } else if (this._delay == false) {
            this._currentShip = "";
            this._currentName = "";
        } else if (this._shipTempKey == null) {
            this._delay = false;
        }
        this._actuallyBought = 0;
    }
}

//Activate hangar.
this.playerBoughtNewShip = function (ship, price) {
    // if ShipVersion is installed this event handler is ineffective: HyperspaceHangar mission screen 
    // will be interrupted by ShipVersion's
    if (this._svEnabled) return;

    // factored the ship storing offer into a separated function so ShipVersion can call it if the 
    // player confirms the acquisition of the new ship
    this.$offerOldShipStorage(ship, price);
}


this.$offerOldShipStorage = function(ship, price) {
    // can't use stored player credit amount and price to set player credits,
    // because this doesn't take the trade-in value into account
    // the displayed trade in amount appears to be
    //            Math.round((player.ship.price * (player.ship.serviceLevel / 100) * 0.75) / 1000, 0) * 1000
    // however, (this._playerCredit + tradeInValue) - price still doesn't reflect actual calced values
    // must also include sale price of any cargo, but only cargo that was able to be offloaded onto the station's market

    // that said, we do need to take away the trade in value from the player if they store the ship, otherwise it's free money

    this._cost = 0;
    if (this._percentPrice == true) {
        this._cost = (this._shipPrice * (this._shipPercent * 0.1));
    } else if (missionVariables.hyperSpace_mission != "COMPLETE") {
        this._cost = this._priceOption * 0.1;
    } else {
        this._cost = this._priceOption * 0.05;
    }

    // we can only offer to store if the player would have enough remaining cash after the trade in amount is removed
    if ((player.credits - this._tradeIn) >= this._cost && this._actuallyBought == 1) {
        mission.runScreen({
                screenID: "HyperspaceHangar",
                model: "[" + JSON.parse(this._currentShip)[1] + "]",
                modelPersonality: JSON.parse(this._currentShip)[13],
                spinModel: true,
                background: {
                    name: "launch_bay_wip5_by_kheng-d7fmvuj.png",
                    height: 480
                },
                title: "Hyperspace Hangar",
                message: "Would you like to store your ship?",
                exitScreen: "GUI_SCREEN_STATUS",
                choicesKey: "hyperSpace_askstore"
			},
            //storing a ship costs time and money, removing handwavium.
            //moved to this location to ensure price can be used as 
            //a parameter.

            this.$askStore, this);
            // the ship storing fee was calculated at the beginning of the function
            mission.addMessageText("Current price: " + formatCredits(this._cost, true, true));
            mission.addMessageText("Current storage time: " + (this._timeOption / 60).toFixed(1) + " minutes");
    } else {
        // not enough credits
        this._delay = false;
        this._currentShip = "";
        this._currentName = "";
    }
    this._actuallyBought = 0;
}
this.$askStore = function (choice) { 
    if (choice == "1_STORE") {
        // take away the trade-in amount if we're storing the ship
        // the storing fee (this._cost) was calculated before offering the storing option
        if (player.credits - this._tradeIn >= this._cost) {
            player.credits -= this._tradeIn;
            // only put the ship into the storage array now
            // this was a workaround for an issue were the array was being updated mysteriously
            // and previously removed passengers were reappearing in the array
            this._shipsStored[this._shipTempKey] = this._currentShip;
            this._currentNames[this._shipTempKey] = this._currentName;
            player.consoleMessage("Stored " + this._currentNames[this._shipTempKey] + ".");
            //don't delete stored ship
            this._shipTempKey = null;
            //activate hangar
            this._currentNames["0_EXIT"] = "Exit";
            player.credits -= this._cost;
            clock.addSeconds(this._timeOption);
            this.$dockedCheck();
        }
        else {
            log("HyperspaceHangar", "Player doesn't have enough credits for the fee");
        }
    } else {
        this._delay = false;
    }
    this._currentShip = "";
    this._currentName = "";
}

//hangar function proper
this.$storeShips = function () {
    //are options available?
    if (this._showOptions == false) {
        if (this._oxpConf_check != false) {
            player.consoleMessage("Options are available through OXPConfig.");
        } else if (this._libConf_check != false) {
            player.consoleMessage("Options are available through Library Config.");
        } else {
            player.consoleMessage("OXPConfig and Library Config not enabled/installed. Cannot provide options.");
        }
    }
    //deactivate hangar if empty
    if (Object.keys(this._currentNames) == "0_EXIT") {
        this._currentNames["0_EXIT"] = "EMPTY";
    }
    //offer mission
    if (galaxyNumber == 2 && missionVariables.hyperSpace_mission == "NOT_ACTIVATED") {
        missionVariables.hyperSpace_mission = "ACTIVATED"
    }
    //not activated unless another ship is bought
    if (this._currentNames["0_EXIT"] == "EMPTY") {
        if (this._showOptions == false) {
            player.consoleMessage("You must purchase an additional ship in order to use the hangar.");
        } else {
            mission.runScreen({
                    screenID: "HyperspaceHangar",
                    background: {
                        name: "launch_bay_wip5_by_kheng-d7fmvuj.png",
                        height: 480
                    },
                    allowInterrupt: true,
                    title: "Hyperspace Hangar",
                    message: "No ships stored.",
                    exitScreen: "GUI_SCREEN_INTERFACES",
                    choicesKey: "hyperSpace_notstored"
                },
                this.$noneStored, this);
            mission.addMessageText("You must purchase an additional ship in order to use the hangar.");
        }
    }
    //no options
    else if (this._showOptions == false) {
        mission.runScreen({
                screenID: "HyperspaceHangar",
                background: {
                    name: "launch_bay_wip5_by_kheng-d7fmvuj.png",
                    height: 480
                },
                allowInterrupt: true,
                title: "Hyperspace Hangar",
                message: "Choose an option.",
                exitScreen: "GUI_SCREEN_INTERFACES",
                choicesKey: "hyperSpace_nooptions"
            },
            this.$choice, this);
    }
    //options
    else {
        mission.runScreen({
                screenID: "HyperspaceHangar",
                background: {
                    name: "launch_bay_wip5_by_kheng-d7fmvuj.png",
                    height: 480
                },
                allowInterrupt: true,
                title: "Hyperspace Hangar",
                message: "Choose an option.",
                exitScreen: "GUI_SCREEN_INTERFACES",
                choicesKey: "hyperSpace_choices"
            },
            this.$choice, this);
    }
}


this.$localizeShips = function () {
    var _systemID = system.ID;
    var k, _ship;

    this._inStationNames = {};
    this._otherStationNames = {};
    log(this.name, "current system: "+_systemID);
    for (k in this._currentNames) {
        if (k != "0_EXIT") {
            _ship = JSON.parse(this._shipsStored[k]);
            if (_systemID == _ship[0][1]) {
                this._inStationNames[k] = _currentNames[k];
                if (this._debugLogging) log(this.name, k+": Ship "+_currentNames[k]+" in current system ("+_ship[0][1]+")");
            } else {
                this._otherStationNames[k] = "(in " +_ship[0][4] + ") " + _currentNames[k];
                if (this._debugLogging) log(this.name, k+": Ship "+_currentNames[k]+" in "+_ship[0][4]+" ("+_ship[0][1]+")");
            }
        }
    }
    if (this._debugLogging) {
        log(this.name, "Ships in system: "+JSON.stringify(this._inStationNames));
        log(this.name, "Ships in other systems: "+JSON.stringify(this._otherStationNames));
    }
}

//menu function, used for almost every main menu, exepting no ships
this.$choice = function (choice) {
    var parameters;

	this.$localizeShips();
    if (choice == "GARAGE_1_SWITCH") {
        parameters = {
                screenID: "HyperspaceHangar",
                background: {
                    name: "launch_bay_wip5_by_kheng-d7fmvuj.png",
                    height: 480
                },
                allowInterrupt: false, 
                title: "Hyperspace Hangar",
                message: "Choose a ship:",
                exitScreen: "GUI_SCREEN_INTERFACES",
                choices: this._inStationNames
        };
        worldScripts["Paginated Screen"].$runScreen(parameters, this.$choice_switch, this, true);
        mission.addMessageText("Current storage time: " + (this._timeOption / 60) + " minutes");
    } else if (choice == "GARAGE_2_SELL") {
        mission.runScreen({
                screenID: "HyperspaceHangar",
                background: {
                    name: "launch_bay_wip5_by_kheng-d7fmvuj.png",
                    height: 480
                },
                allowInterrupt: true,
                title: "Hyperspace Hangar",
                message: "Choose an option.",
                exitScreen: "GUI_SCREEN_INTERFACES",
                choicesKey: "hyperSpace_sellChoices"
            },
            this.$choice_sell, this);
        mission.addMessageText("WARNING: ALL SOLD SHIPS WILL BE PERMANENTLY LOST.");
    } else if (choice == "GARAGE_3_TRANSFER") {
        mission.runScreen({
                screenID: "HyperspaceHangar",
                background: {
                    name: "launch_bay_wip5_by_kheng-d7fmvuj.png",
                    height: 480
                },
                allowInterrupt: true,
                title: "Hyperspace Hangar",
                message: "Choose an option.",
                exitScreen: "GUI_SCREEN_INTERFACES",
                choicesKey: "hyperSpace_transferChoices"
            },
            this.$choice_transfer, this);
    } else if (choice == "GARAGE_4_OPTIONS") {
        mission.runScreen({
                screenID: "HyperspaceHangar",
                background: {
                    name: "launch_bay_wip5_by_kheng-d7fmvuj.png",
                    height: 480
                },
                allowInterrupt: true,
                title: "Hyperspace Hangar",
                message: "Choose an option.",
                exitScreen: "GUI_SCREEN_INTERFACES",
                choicesKey: "hyperSpace_options"
            },
            this.$choice_options, this);
        mission.addMessageText("Current selling time: " + (this._timeOption / 60) + " minutes");
        mission.addMessageText("Current value: " + formatCredits(this._priceOption * 0.1, true, true));
    } else if (choice == "GARAGE_5_INFO") {
        //"normal" info screen
        if (missionVariables.hyperSpace_mission != "COMPLETED") {
            mission.runScreen({
                screenID: "HyperspaceHangar",
                title: "Station Database: Norby-Ramen Quirium Accuracy Drive",
                exitScreen: "GUI_SCREEN_INTERFACES",
                messageKey: "hyperSpace_QAD"
            });
        }
        //complete info screen
        else if (missionVariables.hyperSpace_mission == "COMPLETED") {
            mission.runScreen({
                    screenID: "HyperspaceHangar",
                    title: "Station Database: Norby-Ramen Quirium Accuracy Drive",
                    messageKey: "hyperSpace_QAD_authorized",
                    exitScreen: "GUI_SCREEN_INTERFACES",
                    choicesKey: "hyperSpace_QAD_nextPage"
                },
                this.$info_next_page, this);
        }
    }
}

this.$choice_switch = function (choice) {
    //give chance to exit. ditto for sell function.
    if (choice == "0_EXIT") {
        this.$storeShips();
        return;
    } else if (system.ID != JSON.parse(this._shipsStored[choice])[0][1]) {
        player.consoleMessage("Ship not in hangar. Transfer ship first to switch.");
        this.$storeShips();
        return;
    }
    this._oldChoice = choice;
    //confirm choice with possible damage
    if (player.ship.dockedStation.equivalentTechLevel < 7 && missionVariables.hyperSpace_mission != "COMPLETE") {
        mission.runScreen({
                screenID: "HyperspaceHangar",
                model: "[" + JSON.parse(this._shipsStored[choice])[1] + "]",
                modelPersonality: JSON.parse(this._shipsStored[choice])[13],
                background: {
                    name: "launch_bay_wip5_by_kheng-d7fmvuj.png",
                    height: 480
                },
                title: "Hyperspace Hangar",
                message: "Transferring at the currently docked station may be damaging. Continue anyway?",
                exitScreen: "GUI_SCREEN_INTERFACES",
                choicesKey: "hyperSpace_confirm"
            },
            this.$choice_confirm_damage, this);
    }
    //confirm choice
    else {
        mission.runScreen({
                screenID: "HyperspaceHangar",
                model: "[" + JSON.parse(this._shipsStored[choice])[1] + "]",
                modelPersonality: JSON.parse(this._shipsStored[choice])[13],
                spinModel: true,
                background: {
                    name: "launch_bay_wip5_by_kheng-d7fmvuj.png",
                    height: 480
                },
                title: "Hyperspace Hangar",
                message: "Switch ship to " + this._currentNames[choice] + "?",
                exitScreen: "GUI_SCREEN_STATUS",
                choicesKey: "hyperSpace_confirm"
            },
            this.$choice_confirm_switch, this);
    }
}

//sell menus
this.$choice_sell = function (choice) {
    var parameters;
    if (choice == "1_SELLCURRENT") {
        parameters = {
                screenID: "HyperspaceHangar",
                background: {
                    name: "launch_bay_wip5_by_kheng-d7fmvuj.png",
                    height: 480
                },
                allowInterrupt: false,
                title: "Hyperspace Hangar",
                message: "Choose a ship to switch into after the sale.",
                exitScreen: "GUI_SCREEN_INTERFACES",
                choices: this._inStationNames
            };
        worldScripts["Paginated Screen"].$runScreen(parameters, this.$choice_sell_sub1, this, true);
    } else if (choice == "2_SELLSTORED") {
        var _ships = {};
        var s;
        for (s in this._inStationNames) 
            _ships[s] = "(in current system) "+this._inStationNames[s];
        for (s in this._otherStationNames)
            _ships[s] = this._otherStationNames[s];
        parameters = {
                screenID: "HyperspaceHangar",
                background: {
                    name: "launch_bay_wip5_by_kheng-d7fmvuj.png",
                    height: 480
                },
                allowInterrupt: false,
                title: "Hyperspace Hangar",
                message: "Choose a ship to sell.",
                exitScreen: "GUI_SCREEN_INTERFACES",
                choices: _ships
            };
        worldScripts["Paginated Screen"].$runScreen(parameters, this.$choice_sell_sub2, this, true);
    }
}

this.$choice_sell_sub1 = function (choice) {
    //don't sell your options!
    if (choice == "0_EXIT") {
        this.$storeShips();
        return;
    }
    this._oldChoice = choice;
    mission.runScreen({
            screenID: "HyperspaceHangar",
            model: "[" + player.ship.dataKey + "]",
            modelPersonality: player.ship.entityPersonality,
            spinModel: true,
            background: {
                name: "launch_bay_wip5_by_kheng-d7fmvuj.png",
                height: 480
            },
            title: "Hyperspace Hangar",
            message: "Sell " + player.ship.displayName + "?",
            exitScreen: "GUI_SCREEN_INTERFACES",
            choicesKey: "hyperSpace_confirm"
        },
        this.$choice_confirm_sell1, this);
}

this.$choice_sell_sub2 = function (choice) {
    //don't sell your options!
    if (choice == "0_EXIT") {
        this.$storeShips();
        return;
    }
    this._oldChoice = choice;
    mission.runScreen({
            screenID: "HyperspaceHangar",
            model: "[" + JSON.parse(this._shipsStored[choice])[1] + "]",
            modelPersonality: JSON.parse(this._shipsStored[choice])[13],
            spinModel: true,
            background: {
                name: "launch_bay_wip5_by_kheng-d7fmvuj.png",
                height: 480
            },
            allowInterrupt: true,
            title: "Hyperspace Hangar",
            message: "Sell " + this._currentNames[choice] + "?",
            exitScreen: "GUI_SCREEN_INTERFACES",
            choicesKey: "hyperSpace_confirm"
        },
        this.$choice_confirm_sell2, this);
}

//transfer menu
this.$choice_transfer = function (choice) {
    var parameters;
    if (choice == "1_SEND") {
        var _ships = {};
        var s;
        for (s in this._inStationNames) 
            _ships[s] = "(in current system "+this._inStationNames[s];
        for (s in this._otherStationNames) 
            _ships[s] = this._otherStationNames[s];
        parameters = {
                screenID: "HyperspaceHangar",
                background: {
                    name: "launch_bay_wip5_by_kheng-d7fmvuj.png",
                    height: 480
                },
                allowInterrupt: false,
                title: "Hyperspace Hangar",
                message: "Choose a ship to send to the location currently marked on the map: " + System.systemNameForID(player.ship.targetSystem) + ".",
                exitScreen: "GUI_SCREEN_INTERFACES",
                choices: _ships
            };
        worldScripts["Paginated Screen"].$runScreen(parameters, this.$choice_transfer_send, this, true);
    } else if (choice == "2_ORDER") {
        parameters = {
                screenID: "HyperspaceHangar",
                background: {
                    name: "launch_bay_wip5_by_kheng-d7fmvuj.png",
                    height: 480
                },
                allowInterrupt: false,
                title: "Hyperspace Hangar",
                message: "Have a ship sent to the current docked station.",
                exitScreen: "GUI_SCREEN_INTERFACES",
                choices: this._otherStationNames
            };
        worldScripts["Paginated Screen"].$runScreen(parameters, this.$choice_transfer_order, this, true);
    }
}

this.$choice_transfer_send = function (choice) {
    if (choice == "0_EXIT") {
        this.$storeShips();
        return;
    }
    this._oldChoice = choice;
    this._oldShips = JSON.parse(this._shipsStored[choice]);
    //redundancy check
    if (player.ship.targetSystem == JSON.parse(this._shipsStored[choice])[0][1]) {
        player.consoleMessage("Your ship is already at the selected system.");
        this.$storeShips();
        return;
    }
    //galaxy consistency check
    else if (this._oldShips[0][0] != galaxyNumber) {
        player.consoleMessage("Your ship is in another galaxy. Please order instead.");
        this.$storeShips();
        return;
    }
    //is the short range chart usable?
    if (System.infoForSystem(this._oldShips[0][0], this._oldShips[0][1]).distanceToSystem(System.infoForSystem(galaxyNumber, player.ship.targetSystem)) <= 7.0 &&
        this._oldShips[0][0] == galaxyNumber) {
        mission.runScreen({
                screenID: "HyperspaceHangar-map",
                model: "[" + this._oldShips[1] + "]",
                modelPersonality: this._oldShips[13],
                spinModel: true,
                backgroundSpecial: "SHORT_RANGE_CHART",
                title: "Hyperspace Hangar",
                message: "\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nTransfer " + this._currentNames[choice] + " from " + JSON.parse(this._shipsStored[choice])[0][4] + " to " +
                    System.infoForSystem(galaxyNumber, player.ship.targetSystem).name + "?",
                exitScreen: "GUI_SCREEN_INTERFACES",
                choicesKey: "hyperSpace_confirm"
            },
            this.$choice_confirm_transfer_send, this);
    }
    //what mode is the map in?
    else {
        mission.runScreen({
                screenID: "HyperspaceHangar-map",
                model: "[" + this._oldShips[1] + "]",
                modelPersonality: this._oldShips[13],
                spinModel: true,
                backgroundSpecial: (player.ship.routeMode != "OPTIMIZED_BY_TIME" ? "LONG_RANGE_CHART_SHORTEST" : "LONG_RANGE_CHART_QUICKEST"),
                title: "Hyperspace Hangar",
                message: "\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\nTransfer " + this._currentNames[choice] + " from " + JSON.parse(this._shipsStored[choice])[0][4] + " to " +
                    System.infoForSystem(galaxyNumber, player.ship.targetSystem).name + "?",
                exitScreen: "GUI_SCREEN_INTERFACES",
                choicesKey: "hyperSpace_confirm"
            },
            this.$choice_confirm_transfer_send, this);
    } 
    mission.addMessageText("Current transfer price: " +
        formatCredits(System.infoForSystem(this._oldShips[0][0], this._oldShips[0][1]).routeToSystem(System.infoForSystem(galaxyNumber, player.ship.targetSystem)).distance * 90, true, true));
}

this.$choice_transfer_order = function (choice) {
    if (choice == "0_EXIT") {
        this.$storeShips();
        return;
    }
    this._oldChoice = choice;
    this._oldShips = JSON.parse(this._shipsStored[choice]);
    //redundancy check
    if (this._oldShips[0][1] == system.ID) {
        player.consoleMessage("Ship already present.");
        this.$storeShips();
        return;
    }
    //intergalactic order 
    else if (this._oldShips[0][0] != galaxyNumber) {
        mission.runScreen({
                screenID: "HyperspaceHangar",
                model: "[" + this._oldShips[1] + "]",
                modelPersonality: this._oldShips[13],
                spinModel: true,
                background: {
                    name: "launch_bay_wip5_by_kheng-d7fmvuj.png",
                    height: 480
                },
                title: "Hyperspace Hangar",
                message: "Transfer " + this._currentNames[choice] + " to Galaxy " + (galaxyNumber + 1) + "?",
                exitScreen: "GUI_SCREEN_INTERFACES",
                choicesKey: "hyperSpace_confirm"
            },
            this.$choice_confirm_transfer_order_interGalactic, this);
        mission.addMessageText("Current location: " + System.infoForSystem(this._oldShips[0][0], this._oldShips[0][1]).name);
        mission.addMessageText("Current order cost: " + formatCredits(Math.abs(galaxyNumber - this._oldShips[0][0]) * 1000, true, true));
        mission.addMessageText("Current shipping time: " + Math.abs(galaxyNumber - this._oldShips[0][0]) * 72 + " hours");
    }
    //regular order
    else {
        mission.runScreen({
                screenID: "HyperspaceHangar",
                model: "[" + this._oldShips[1] + "]",
                modelPersonality: this._oldShips[13],
                spinModel: true,
                background: {
                    name: "launch_bay_wip5_by_kheng-d7fmvuj.png",
                    height: 480
                },
                title: "Hyperspace Hangar",
                message: "Transfer " + this._currentNames[choice] + " to " + system.name + "?",
                exitScreen: "GUI_SCREEN_INTERFACES",
                choicesKey: "hyperSpace_confirm"
            },
            this.$choice_confirm_transfer_order, this);
        mission.addMessageText("Current location: " + System.infoForSystem(this._oldShips[0][0], this._oldShips[0][1]).name);
        mission.addMessageText("Current order cost: " + formatCredits(system.info.routeToSystem(System.infoForSystem(this._oldShips[0][0], this._oldShips[0][1]), player.ship.routeMode).distance * 90, true, true));
        mission.addMessageText("Current shipping time: " + system.info.routeToSystem(System.infoForSystem(this._oldShips[0][0], this._oldShips[0][1]), player.ship.routeMode).time.toFixed(0) + " hours");
    }
}

this.$choice_options = function (choice) {
    if (choice == "1_PRICE") {
        if (this._percentPrice == true) {
            player.consoleMessage("Standard pricing disabled. Check " + this._configType + ". Current value: " + this._shipPercent * 10 + "%.");
        } else {
            mission.runScreen({
                    screenID: "HyperspaceHangar",
                    background: {
                        name: "launch_bay_wip5_by_kheng-d7fmvuj.png",
                        height: 480
                    },
                    allowInterrupt: true,
                    title: "Hyperspace Hangar",
                    message: "Input price",
                    exitScreen: "GUI_SCREEN_INTERFACES",
                    textEntry: "TRUE"
                },
                this.$choice_price, this);
            mission.addMessageText("Current value: " + formatCredits(this._priceOption / 10, true, true));
        }
    } else if (choice == "2_TIME") {
        mission.runScreen({
                screenID: "HyperspaceHangar",
                background: {
                    name: "launch_bay_wip5_by_kheng-d7fmvuj.png",
                    height: 480
                },
                allowInterrupt: true,
                title: "Hyperspace Hangar",
                message: "Input time (in seconds)",
                exitScreen: "GUI_SCREEN_INTERFACES",
                textEntry: "TRUE"
            },
            this.$choice_time, this);
        mission.addMessageText("Current value: " + (this._timeOption / 60) + " minutes");
    } else if (choice == "3_RESET") {
        this._priceOption = this._defaultpriceOption;
        this._timeOption = this._defaulttimeOption;
    }
    //removes ships entirely
    else if (choice == "4_PURGE") {
        var _ships = {};
        var s;
        for (s in this._inStationNames)
            _ships[s] = "(in current system) "+this._inStationNames[s];
        for (s in this._otherStationNames)
            _ships[s] = this._otherStationNames[s];
        parameters = {
                screenID: "HyperspaceHangar",
                background: {
                    name: "launch_bay_wip5_by_kheng-d7fmvuj.png",
                    height: 480
                },
                allowInterrupt: false,
                title: "Hyperspace Hangar",
                message: "Choose ship to remove. WARNING: REMOVAL IS PERMANENT!",
                exitScreen: "GUI_SCREEN_INTERFACES",
                choices: _ships
            };
        worldScripts["Paginated Screen"].$runScreen(parameters, this.$purge, this, true);
    }
}

this.$choice_price = function (choice) {
    this._priceOption = choice * 10;
}

this.$choice_time = function (choice) {
    this._timeOption = choice;
}

this.$purge = function (choice) {
    if (choice == "0_EXIT") {
        this.$storeShips();
        return;
    }
    this._oldChoice = choice;
    mission.runScreen({
            screenID: "HyperspaceHangar",
            model: "[" + JSON.parse(this._shipsStored[choice])[1] + "]",
            modelPersonality: JSON.parse(this._shipsStored[choice])[13],
            spinModel: true,
            background: {
                name: "launch_bay_wip5_by_kheng-d7fmvuj.png",
                height: 480
            },
            title: "Hyperspace Hangar",
            message: "Remove " + this._currentNames[choice] + "?",
            choicesKey: "hyperSpace_confirm"
        },
        this.$choice_confirm_purge, this);
}

//menu for no ships
this.$noneStored = function (choice) {
    if (choice == "1_OPTIONS") {
        mission.runScreen({
                screenID: "HyperspaceHangar",
                background: {
                    name: "launch_bay_wip5_by_kheng-d7fmvuj.png",
                    height: 480
                },
                allowInterrupt: true,
                title: "Hyperspace Hangar",
                message: "Choose an option.",
                exitScreen: "GUI_SCREEN_INTERFACES",
                choicesKey: "hyperSpace_options"
            },
            this.$choice_options, this);
    } else if (choice == "2_INFO") {
        if (missionVariables.hyperSpace_mission != "COMPLETED") {
            mission.runScreen({
                screenID: "HyperspaceHangar",
                title: "Station Database: Norby-Ramen Quirium Accuracy Drive",
                exitScreen: "GUI_SCREEN_INTERFACES",
                messageKey: "hyperSpace_QAD"
            });
        } else if (missionVariables.hyperSpace_mission == "COMPLETED") {
            mission.runScreen({
                    screenID: "HyperspaceHangar",
                    title: "Station Database: Norby-Ramen Quirium Accuracy Drive",
                    messageKey: "hyperSpace_QAD_authorized",
                    exitScreen: "GUI_SCREEN_INTERFACES",
                    choices: ({
                        "1_NEXT": "Next page"
                    })
                },
                this.$info_next_page, this);
        }
    }
}

this.$choice_confirm_switch = function (choice) {
    if (choice == "1_YES") {
        //switch ships
        var hud = player.ship.hud;
        var credits = player.credits;
        var shipinfo = worldScripts["Ship_Storage_Helper.js"].storeCurrentShip(player.ship);
        var data = JSON.parse(shipinfo);
        var key = clock.absoluteSeconds;
        data[8].length = 0; // remove cargo
        data[10].length = 0; // remove passengers
        data[31] = ""; // remove cargo in smuggling compartment
        this._shipsStored[key] = JSON.stringify(data);
        this._currentNames[key] = player.ship.displayName;
        if (this._svEnabled) { worldScripts.shipversion.$hyperspaceHangarRestoredShip = true; }
        try {
            worldScripts["Ship_Storage_Helper.js"].restoreStoredShip(this._shipsStored[this._oldChoice]);
        } catch (err) {
            log(this.name, "!!ERROR: " + err);
        }
        player.credits = credits;
        player.ship.hud = hud;

        //don't keep switched ship stored!
        player.consoleMessage("Switched ship to " + this._currentNames[this._oldChoice] + ".");
        delete this._shipsStored[this._oldChoice];
        delete this._currentNames[this._oldChoice];
        this._oldChoice = "";
        clock.addSeconds(this._timeOption);
    } else this.$storeShips();
}

this.$choice_confirm_damage = function (choice) {
    if (choice == "1_YES") {
        //switch ships
        var hud = player.ship.hud;
        var credits = player.credits;
        var shipinfo = worldScripts["Ship_Storage_Helper.js"].storeCurrentShip(player.ship);
        var data = JSON.parse(shipinfo);
        var key = clock.absoluteSeconds;
        data[8].length = 0; // remove cargo
        data[10].length = 0; // remove passengers
        data[31] = ""; // remove in smuggling compartment
        var shipinfo2 = JSON.stringify(data);
        this._shipsStored[key] = shipinfo2;
        this._currentNames[key] = player.ship.displayName;
        if (this._svEnabled) { worldScripts.shipversion.$hyperspaceHangarRestoredShip = true; }
        try {
            worldScripts["Ship_Storage_Helper.js"].restoreStoredShip(this._shipsStored[this._oldChoice]);
        } catch (err) {
            log(this.name, "!!ERROR: " + err);
        }
        player.credits = credits;
        player.ship.hud = hud;

        //don't keep switched ship stored!
        player.consoleMessage("Switched ship to " + this._currentNames[this._oldChoice] + ".");
        delete this._shipsStored[this._oldChoice];
        delete this._currentNames[this._oldChoice];
        this._oldChoice = "";
        clock.addSeconds(this._timeOption);

        //possibly damage ship
        player.ship.takeInternalDamage();
        if (player.ship.takeInternalDamage() == true) {
            player.consoleMessage("Ship damaged during switch.");
        }
    } else this.$storeShips();
}

this.$choice_confirm_sell1 = function (choice) {
    if (choice == "1_YES") {
        //"sell" ship
        var hud = player.ship.hud;
        var cost = this.$tradeInAmount();
        var old_name = player.ship.displayName;
        player.credits += cost;
        try {
            worldScripts["Ship_Storage_Helper.js"].restoreStoredShip(this._shipsStored[this._oldChoice]);
        } catch (err) {
            log(this.name, "!!ERROR: " + err);
        }
        player.ship.hud = hud;
        player.consoleMessage("Sold " + old_name + " for " + formatCredits(cost, false, true) + ".");
        log(this.name, "Sold  " + old_name + " for " + formatCredits(cost, false, true) + ".");
        delete this._shipsStored[this._oldChoice];
        delete this._currentNames[this._oldChoice];
        this._oldChoice = "";
        clock.addSeconds(this._timeOption);
    } else this.$storeShips();
}

this.$choice_confirm_sell2 = function (choice) {
    if (choice == "1_YES") {
        //store ship
        var hud = player.ship.hud;
        //this._shipTempKey = clock.absoluteSeconds;
        var holdShip = worldScripts["Ship_Storage_Helper.js"].storeCurrentShip(player.ship);
        
        //reload ship for pricing
        if (worldScripts.ShipConfiguration_Core) {
            worldScripts.ShipConfiguration_Core._resprayActive = true;
            worldScripts.ShipConfiguration_Core._adding = true;
        }
        try {
            worldScripts["Ship_Storage_Helper.js"].restoreStoredShip(this._shipsStored[this._oldChoice]);
        } catch (err) {
            log(this.name, "!!ERROR: " + err);
        }
        //log(this.name, "current ship " + player.ship.displayName + ", cost " + player.ship.price);
        var cost = this.$tradeInAmount();
        player.credits += cost;
        player.consoleMessage("Sold " + this._currentNames[this._oldChoice] + " for " + formatCredits(cost, false, true) + ".");
        delete this._shipsStored[this._oldChoice];
        delete this._currentNames[this._oldChoice];
        this._oldChoice = "";

        //reload original ship
        try {
            worldScripts["Ship_Storage_Helper.js"].restoreStoredShip(holdShip);
        } catch (err) {
            log(this.name, "!!ERROR: " + err);
        }
        if (worldScripts.ShipConfiguration_Core) {
            worldScripts.ShipConfiguration_Core._resprayActive = false;
            worldScripts.ShipConfiguration_Core._adding = false;
        }
        player.ship.hud = hud;
        clock.addSeconds(this._timeOption);
        this._confirm = false;
    } else this.$storeShips();
}

this.$choice_confirm_transfer_send = function (choice) {
    if (choice == "1_YES") {
        var _fee = (System.infoForSystem(this._oldShips[0][0], this._oldShips[0][1]).routeToSystem(System.infoForSystem(galaxyNumber, player.ship.targetSystem, player.ship.routeMode)).distance * 90);
        log(this.name, "Player confirmed transfer of ship "+this._currentNames[this._oldChoice]+" from "+this._oldShips[0][4]+" to "+System.infoForSystem(galaxyNumber, player.ship.targetSystem).name+" for "+formatCredits(_fee, true, true));
        this._oldShips = JSON.parse(this._shipsStored[this._oldChoice]);
        //subtract money
        log(this.name, "Player confirmed transfer of ship "+this._currentNames[this._oldChoice]+" from "+this._oldShips[0][4]+" to "+System.infoForSystem(galaxyNumber, player.ship.targetSystem).name+" for "+formatCredits(_fee, true, true));
        player.credits -= _fee;
        //change values
        this._oldShips[0][0] = galaxyNumber;
        this._oldShips[0][1] = player.ship.targetSystem;
        this._oldShips[0][4] = System.infoForSystem(galaxyNumber, player.ship.targetSystem).name;
        this._shipsStored[this._oldChoice] = JSON.stringify(this._oldShips);
        //give feedback
        player.consoleMessage(this._oldShips[0][3] + " sent to " + System.infoForSystem(this._oldShips[0][0], this._oldShips[0][1]).name + ".");
        this.$localizeShips();
    }
    this.$choice_transfer("1_SEND");
}

this.$choice_confirm_transfer_order = function (choice) {
    if (choice == "1_YES") {
        this._oldShips = JSON.parse(this._shipsStored[this._oldChoice]);
        //subtract money
        player.credits -= system.info.routeToSystem(System.infoForSystem(this._oldShips[0][0], this._oldShips[0][1], player.ship.routeMode)).distance * 90;
        //add time
        clock.addSeconds(system.info.routeToSystem(System.infoForSystem(this._oldShips[0][0], this._oldShips[0][1]), player.ship.routeMode).time * 3600);
        //give feedback
        player.consoleMessage("Got " + this._oldShips[0][3] + " from " + System.infoForSystem(this._oldShips[0][0], this._oldShips[0][1]).name + ".");
        //change values
        this._oldShips[0][0] = galaxyNumber;
        this._oldShips[0][1] = system.ID;
        this._oldShips[0][4] = system.name;
        this._shipsStored[this._oldChoice] = JSON.stringify(this._oldShips);
        this.$localizeShips();
    }
    this.$choice_transfer("2_ORDER");
}

this.$choice_confirm_transfer_order_interGalactic = function (choice) {
    if (choice == "1_YES") {
        //subtract money
        player.credits -= Math.abs(galaxyNumber - this._oldShips[0][0]) * 900;
        //add time
        clock.addSeconds(Math.abs(galaxyNumber - this._oldShips[0][0]) * 72 * 3600);
        //give feedback
        player.consoleMessage("Got " + this._oldShips[0][3] + " from " + System.infoForSystem(this._oldShips[0][0], this._oldShips[0][1]).name + ".");
        //change values
        this._oldShips[0][0] = galaxyNumber;
        this._oldShips[0][1] = system.ID;
        this._oldShips[0][4] = system.name;
        this._shipsStored[this._oldChoice] = JSON.stringify(this._oldShips);
    } else this.$storeShips();
}


this.$choice_confirm_purge = function (choice) {
    if (choice == "1_YES") {
        player.consoleMessage("Removed " + this._currentNames[this._oldChoice] + ".");
        delete this._currentNames[this._oldChoice];
        delete this._shipsStored[this._oldChoice];
        this._oldChoice = "";
    } else this.$storeShips();
}

//next page of info for mission completed info screen
this.$info_next_page = function (choice) {
    mission.runScreen({
        screenID: "HyperspaceHangar",
        title: "Station Database: Norby-Ramen Quirium Accuracy Drive",
        exitScreen: "GUI_SCREEN_INTERFACES",
        messageKey: "hyperSpace_QAD_authorized2"
    });
}

// clean out the stored ships dictionary, leaving only those ships which are actually stored
this.$cleanUp = function $cleanUp () {
    var list = Object.keys(this._currentNames);
    var full = Object.keys(this._shipsStored);
    for (var i = 0; i < full.length; i++) {
        if (list.indexOf(full[i]) === -1) {
            delete this._shipsStored[full[i]];
        }
    }
}

this.$tradeInAmount = function $tradeInAmount() {
    var p = player.ship;
    var amt = Math.round(p.price * (p.serviceLevel / 100) * (0.75) / 1000) * 1000;
    return amt;
}
