"use strict";
this.name = "Paginated Screen";
this.author = "Dybal";
this.licence = "CC-BY-NC-SA 3.0";
this.description = "Provides a paginated mission screen" 

this._hudHidden;
this._maxContentRows = 21;
this._msRows = 23;
this._itemColor = "greenColor";
this._menuColor = "orangeColor";
this._maxPage;
this._curPage;
this._contentChoices = [];
this._contentCallback;
this._contentThisObj;
this._contentParameters;
this._contentExitChoice;

this.$runScreen = function _runPaginatedScreen(parameters, callback, this_obj, sort_entries) {
    this._contentCallback = callback;
    this._contentThisObj = this_obj;
    this._contentParameters = parameters;

    log(this.name, "original choices: "+JSON.stringify(parameters.choices));
    var _ship = player.ship;
    this.$hudHidden = player.ship.hudHidden;
    player.ship.hudHidden = true;

    if (!parameters.choices) return "'choices' not found in 'parameters'";
    var k;
    var original_choices = parameters.choices;
    this._contentChoices = [];
    for (k in original_choices) {
        if ((k.toUpperCase()).indexOf("EXIT") > -1) {
            this._contentExitChoice = k;
            log(this.name, "Original Exit choice: "+this._contentExitChoice);
        } else {
            this._contentChoices.push({key:k, value:original_choices[k]});
        }
    }
    if (sort_entries) {
        log(this.name, "sorting entries");
        this._contentChoices.sort(this._compareEntries);
    }

    if (!this._contentExitChoice) this._contentExitChoice = "0_EXIT";
    log(this.name, "Exit choice: "+this._contentExitChoice);
    this._maxPage = Math.ceil(this._contentChoices.length / this._maxContentRows)-1;
    if (this._maxPage < 0) this._maxPage = 0; // if original choices is empty...
    this._curPage = 0;
    log(this.name, this._contentChoices.length+" entries, "+this._maxPage+" pages");
    this._showPage(0);
}

this._compareEntries = function(a,b) {
    var _a, _b;
    if (typeof a.value === 'object' && a.value.text) {
        _a = a.value.text;
    } else {
        _a = a.value;
    }
    if (typeof b.value === 'object' && b.value.text) {
        _b = b.value.text;
    } else {
        _b = b.value;
    }
    return (_a > _b);    
}


this._showPage = function(page) {
    if (page < 0) { this._curPage = 0; }
    else if (page > this._maxPage) { this._curPage = this._maxPage }
    else { this._curPage = page; }
  
    var j;
    for (j=0; j < this._contentChoices.length; j++) {
        log(this.name, j+": "+this._contentChoices[j].key+": "+this._contentChoices[j].value);
    }

    var _first_line = this._curPage * this._maxContentRows;
    var choices = {};
    var i = 0;
    var c, _key, _padding, j;
    var _zero = "0";
 
    log(this.name, "Showing page "+page+", starts at "+_first_line+", pages with "+this._maxContentRows+" lines");
    while (i < this._maxContentRows && _first_line + i < this._contentChoices.length ) {
        c = this._contentChoices[_first_line+i];
        _key = (_first_line+i).toString();
        if (_key.length < 4) {
            j = 4 - _key.length;
            while (j--) {
                _key = "0" + _key;
            }
            _key = "C_"+ _key;
        }
        choices[_key] = this._formatChoice(c.value);
        i++;
    }
    while (i < this._msRows) {
        choices['Z96_SPACER_'+i] = "";
        i++;
    }
    if (this._curPage > 0) { 
        choices['Z97_PREVIOUS_PAGE'] = {text:"Previous Page", color:this._menuColor, alignment:"CENTER"}; 
    }
    if (this._curPage < this._maxPage ) { 
        choices['Z98_NEXT_PAGE'] = {text:"Next Page", color:this._menuColor, alignment:"CENTER"}; 
    }
    choices['Z99_EXIT'] = {text:"Exit", color:this._menuColor, alignment:"CENTER"};
    this._contentParameters.choices = choices;

    var k;
    var c;
    var i = 0;
    for (k in this._contentParameters.choices) {
        c = this._contentParameters.choices[k];
        log(this.name, "linha "+i+": "+k+":{text:"+c.text+"}"); 
        i++;
    }

    player.ship.hudHidden = true;
    mission.runScreen(this._contentParameters, this._pChoice, this);
}


this._formatChoice = function (choice) {
    if (typeof choice === "object") {
        return { text: choice.text, color:(choice.color? choice.color : this._itemColor), alignment: (choice.algnment ? choice.alignment : "LEFT")};
    } else {
        return { text: choice, color:this._itemColor, alignment:"LEFT"} ;
    }
}


this._pChoice = function (choice) {
    log(this.name, "choice: "+choice);
    if (choice === 'Z97_PREVIOUS_PAGE') {
        this._showPage(this._curPage - 1);
    }
    else if (choice === 'Z98_NEXT_PAGE') {
        this._showPage(this._curPage + 1);
    }
    else if (choice === "Z99_EXIT") {
        player.ship.hudHidden = this._hudHidden;
        this._contentCallback.call(this._contentThisObj, this._contentExitChoice);
    }
    else {
        var c;
        c = choice.replace("C_", "") * 1;
        log(this.name, choice+"("+c +"): "+this._contentChoices[c].key+" ("+this._contentChoices[c].value+") chosen");
        player.ship.hudHidden = this._hudHidden;
        this._contentCallback.call(this._contentThisObj, this._contentChoices[c].key);
    }
}
